/**
 * 
 */
package projectile;

import java.io.*;

public aspect LogAspect {
	
	private int numTirs = 1;

	//// Inici tir
	pointcut firePointCut(Projectile p) : 
		execution(void Projectile.fire()) && this(p); 
	
	before(Projectile p): firePointCut(p)
	{
		double velocitat = p.velocityInit;
		double angle = p.angle;
		double massa = p.m;
		double resistencia = p.k;
		escriureIniciLog(velocitat, angle, massa, resistencia);
	}
	
	//// Final tir
	pointcut runPointCut(Projectile p) :
		execution(void Projectile.run()) && this(p);
	
	after(Projectile p): runPointCut(p) 
	{
		if (p.hit) {
			double distancia = p.x*p.x0;
			double velocitat = Math.sqrt(p.vx*p.vx*p.vx0*p.vx0+p.vy*p.vy*p.vy0*p.vy0);
			double alcadaMax = p.maxHeight*p.y0;
			double tempsFinal = p.t*p.t0;
			escriureFinalLog(distancia, velocitat, alcadaMax, tempsFinal);
		}
	}

	// Escriure LOG de les dades inicials del tir
	private void escriureIniciLog(double velocitat, double angle, double massa, double resistencia)
	{
		PrintWriter output = obrirFitxerLog("Log.txt");
		
	    output.println("-----------------------------------------------------");
	    output.println("Inici Tir #" + numTirs);
	    output.println("\t- Velocitat = " + velocitat);
	    output.println("\t- Angle = " + angle);
	    output.println("\t- Massa = " + massa);
	    output.println("\t- Resistencia = " + resistencia);
	    output.println("");
	    output.close();
	}
	
	// Escriure LOG de les dades finals del tir
	private void escriureFinalLog(double distancia, double velocitat, double alcadaMax, double tempsFinal)
	{
		PrintWriter output = obrirFitxerLog("Log.txt");
		
	    output.println("Final Tir #" + numTirs);
	    output.println("\t- Distancia final = " + distancia);
	    output.println("\t- Velocitat final = " + velocitat);
	    output.println("\t- Alçada maxima = " + alcadaMax);
	    output.println("\t- Temps final = " + tempsFinal);
	    output.println("");
	    output.close();
	    numTirs++;
	}
	
	// Obre un fitxer per afegir al final, si no existeix el crea
	private PrintWriter obrirFitxerLog(String nomFitxer)
	{
		try
		{
			PrintWriter output;
			boolean append = true;
			FileOutputStream fitxerLog = new FileOutputStream(nomFitxer, append); 
			OutputStreamWriter outputStream =new OutputStreamWriter(fitxerLog);
			BufferedWriter writerLog = new BufferedWriter(outputStream);
			output = new PrintWriter(writerLog);
			
			return output;
		}
		catch(Exception ex)
		{
			System.out.println("LogAspect: Impossible obrir fitxer log!");
			ex.printStackTrace();
			return null;
		}
	}
	
}
